#include "sta310.h"
#include <i2c.h>
#include <p18f6722.h>

void WriteSta(unsigned char addr, unsigned char data)
{
	StartI2C1();
	IdleI2C1();
	WriteI2C1(0xB8);
	IdleI2C1();
	WriteI2C1(addr);
	IdleI2C1();
	WriteI2C1(data);
	IdleI2C1();
	StopI2C1();
}

unsigned char ReadSta(unsigned char addr)
{
	unsigned char tmp;
	StartI2C1();
	IdleI2C1();
	WriteI2C1(0xB8);
	IdleI2C1();
	WriteI2C1(addr);
	IdleI2C1();
	RestartI2C1();
	IdleI2C1();	
	WriteI2C1(0xB9);
	IdleI2C1();
	tmp=ReadI2C();
	StopI2C1();
	return tmp;
}

void SoftReset()
{								//step 0
	WriteSta(0x10,0x01); 		//reset
	WriteSta(0x2B,0x08); 		//breakpoint
	WriteSta(0x3A,0x00); 		//clockcmd
	while(ReadSta(0xFF)!=1); 	//wait till boot
	WriteSta(0xB5,0x01); 		//enable PLL
}

								//step 1 - nothing to be done

void StaAudioPLL()				//step 2, PCMLCK=256xFs
{								//from getting started guide
	WriteSta(0xB6,0x34);		//FRACL coefficient for 256x192 khz
	WriteSta(0xB7,0xEC);		//FRACH coefficient for 256x192 khz
	WriteSta(0xB8,0x02);		//X coefficient for 256x192 khz
	WriteSta(0xB9,0x09);		//M coefficient for 256x192 khz
	WriteSta(0xBA,0x01);		//Configure N coefficient for 256x192 khz
	WriteSta(0xBB,0x03);		//FRACL coefficient for 256x176.4 khz
	WriteSta(0xBC,0x09);		//FRACH coefficient for 256x176.4 khz
	WriteSta(0xBD,0x02);		//X coefficient for 256x176.4 khz
	WriteSta(0xBE,0x09);		//M coefficient for 256x176.4 khz
	WriteSta(0xBF,0x01);		//N coefficient for 256x176.4 khz
}

void StaIntSync()				//step 3
{
	WriteSta(0x07,0xFF);		//enable interrupts 0-7
	WriteSta(0x08,0x0F);		//enable interrupts 8-11 - for STA310 v1
								//for STA310 v2 with autodetect 0x3f@0x08
	WriteSta(0x4F,0x00);		//pcket lock - when first sync word 
	WriteSta(0x50,0x00);		//disable audio ID
	WriteSta(0x53,0x00);		//sync lock
}

void StaInput()					//step 4
{
	WriteSta(0x0C,0x0B);		//input data setup
	WriteSta(0x0D,0x00);		//A/D converter setup
	WriteSta(0x12,0x16);		//Audio clock from S/PDIF, system from PLL
}

void StaOutput()				//step 5
{
	WriteSta(0x55,0x23);		//pcm I2S mode
	WriteSta(0x54,0x01);		//pcm divider
	WriteSta(0x5E,0x00);		//disable SPDiF out
	WriteSta(0xAE,0x00);		//disable vcr??
}

void StaDecoder(unsigned char x)//step 6
{
	switch(x)
		{
		case 0:
			{
			WriteSta(0x4C,0x03);	//PCM
			WriteSta(0x4D,0x03);
			WriteSta(0x05,0x01);	//Fs=44.1 kHz
			WriteSta(0x6F,0x00);	//no downimx
			WriteSta(0xA8,0x01);	//DVD Standart specification
			WriteSta(0xA9,0x00);	//Stereo PCM
			break;
			}
		case 1:
			{
			WriteSta(0x4C,0x05);	//AC3
			WriteSta(0x4D,0x00);
			WriteSta(0x68,0x01);	//enable LFE channel
			WriteSta(0x69,0x02);	//compresion for lineout
			WriteSta(0x6A,0x00);	//disable high level compression
			WriteSta(0x6B,0x00);	//disable low leve amplification
			WriteSta(0x6C,0x00);	//mute if CRC error
			WriteSta(0x6D,0x00);	//default karaoke implementation
			WriteSta(0x6E,0x00);	//stereo output if downmix
			WriteSta(0x6F,0x07);	//5.1 channel configuration
			break;
			}
		case 2:
			{
			WriteSta(0x4C,0x03);	//DTS
			WriteSta(0x4D,0x06);
			WriteSta(0x68,0x01);	//enable LFE channel
			WriteSta(0x69,0x00);	//14-bit mode bitstream
			WriteSta(0x6A,0x00);	//disable dynamic range control
			WriteSta(0x6B,0x00);	//channel A is selected on the center channel
			WriteSta(0x6F,0x07);	//5.1 channel configuration
			break;
			}
		}		
}

void StaPost()					//step 7
{	
	WriteSta(0x57,0x00);		//no delays for all channels
	WriteSta(0x58,0x00);
	WriteSta(0x59,0x00);
	WriteSta(0x5A,0x00);
	WriteSta(0x5B,0x00);
	WriteSta(0x5C,0x00);
	WriteSta(0xAF,0x00);
	WriteSta(0xB0,0x00);
	WriteSta(0x5D,0x01);		//update delays	
	WriteSta(0x65,0x00);		//no prologic
	WriteSta(0x62,0x00);		//no sound processors
	WriteSta(0xB1,0x00);
	WriteSta(0x64,0x04);		//disable srs
	WriteSta(0xAB,0x00);
	WriteSta(0xB2,0x00);		//reset circle sourround mode
	WriteSta(0xB3,0x00);		//no delays for circle surround	
	WriteSta(0x66,0x00);		//no bass
}

void StaStart()					//step 8
{
	WriteSta(0x13,0x01);		//play
	WriteSta(0x14,0x00);		//unmute
	WriteSta(0x72,0x01);		//run decoder
}


